/**************************************************************************************
Author:         Dave Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Date:           11/19/2009
Program:        materials_standard
Description:    This script allows the user to affect muliple std materials
                simultaneously
***************************************************************************************/

try (destroyDialog standard_material_rollout) catch()


struct MaxStandardMaterials
(
    std_materials = #(),
    std_material_names = #(),
    std_numeric_properties = #(),
    std_boolean_properties = #(),
    std_color_properties = #(),
    
    --Map properties have to be hard coded because of the way the standard material is coded, thanks autodesk!
    std_map_properties = #("ambientMap","bumpMap","diffuseMap","displacementMap",
        "filterMap","glossinessMap","opacityMap","reflectionMap",
        "refractionMap","selfIllumMap","specularLevelMap","specularMap"),
    std_map_multiplier_properties = #("ambientMapAmount","bumpMapAmount","diffuseMapAmount","displacementMapAmount",
        "filterMapAmount","glossinessMapAmount","opacityMapAmount","reflectionMapAmount",
        "refractionMapAmount","selfIllumMapAmount","specularLevelMapAmount","specularMapAmount"),
    std_map_boolean_properties = #("ambientMapEnable","bumpMapEnable","diffuseMapEnable","displacementMapEnable",
        "filterMapEnable","glossinessMapEnable","opacityMapEnable","reflectionMapEnable",
        "refractionMapEnable","selfIllumMapEnable","specularLevelMapEnable","specularMapEnable"),
    
    /**************************************************************************
    Sorts items by name
    **************************************************************************/
    fn sort_by_name item_one item_two =
    (
        case of
        (
            (item_one.name < item_two.name): -1
            (item_one.name > item_two.name): 1
            default: 0
        ) --end case
    ), --end fn
    
    /**************************************************************************
    Collects a list of all std materials and their names in the current scene
    **************************************************************************/
    fn collect_standard_materials =
    (
        this.std_materials = getClassInstances Standardmaterial
        
        qsort this.std_materials this.sort_by_name
        
        this.std_material_names = for current in this.std_materials collect current.name
    ), --end fn
    
    /**************************************************************************
    Collects a list of all standard material properties and sorts them
    **************************************************************************/
    fn collect_standard_properties =
    (
        local temp_material = Standardmaterial()
        
        --CRITICAL! Delete previous data!
        this.std_numeric_properties = #()
        this.std_boolean_properties = #()
        this.std_color_properties = #()
        
        for current_property in (getPropNames temp_material) do
        (
            if not (matchPattern current_property pattern:"map*" ignorecase:true) then
            (
                case (classOf (getProperty temp_material current_property)) of
                (
                    (Integer): append std_numeric_properties (current_property as string)
                    (Float): append std_numeric_properties (current_property as string)
                    (BooleanClass): append std_boolean_properties (current_property as string)
                    (Color): append std_color_properties (current_property as string)
                    (String): undefined
                    (UndefinedClass): undefined
                    default: print("WARNING: Unknown property found: " + (current_property as string))
                                
                ) --end case
            ) --end else
        ) --end for
        
        sort this.std_numeric_properties
        sort this.std_boolean_properties
        sort this.std_color_properties
    ), --end fn
    
    /**************************************************************************
    Sets the material property 'property_name' to the value of 'property_value'
    for the materials defined by 'mode'
    **************************************************************************/
    fn set_material_property mode user_selection property_name property_value =
    (
        case mode of
        (
            (1): for index in user_selection do
                    setProperty this.std_materials[index] property_name property_value
            (2): for current in selection do
                    if current.material != undefined do
                        if (classOf current.material) == Standardmaterial do
                            setProperty current.material property_name property_value
            (3): for current in this.std_materials do
                    setProperty current property_name property_value
        ) --end case
    ) --end fn
) --end struct


( --local scope
    global standard_material_rollout
    local std_material_ini = getDir #plugcfg + "\\MaxStandardMaterials.ini"
    local std_material_position
    
    rollout standard_material_rollout "Standard Materials"
    (
        local std_mats = MaxStandardMaterials()
        
        /***********************************************************************
        Enables/Disables the UI controls based on argument 'state'
        ***********************************************************************/
        fn enable_controls state =
        (
            standard_material_rollout.numeric_properties_dropdown.enabled = state
            standard_material_rollout.numeric_properties_spinner.enabled = state
            standard_material_rollout.boolean_properties_dropdown.enabled = state
            standard_material_rollout.boolean_true_button.enabled = state
            standard_material_rollout.boolean_false_button.enabled = state
            standard_material_rollout.color_properties_dropdown.enabled = state
            standard_material_rollout.color_properties_picker.enabled = state
            standard_material_rollout.map_properties_dropdown.enabled = state
            standard_material_rollout.map_properties_button.enabled = state
            standard_material_rollout.map_multiplier_dropdown.enabled = state
            standard_material_rollout.map_multiplier_spinner.enabled = state
            standard_material_rollout.map_boolean_dropdown.enabled = state
            standard_material_rollout.map_boolean_true_button.enabled = state
            standard_material_rollout.map_boolean_false_button.enabled = state
        ) --end fn
        
        /***********************************************************************
        Refreshes the material names in the list box
        ***********************************************************************/
        fn refresh_standard_material_list =
        (
            std_mats.collect_standard_materials()
            
            message = "Total: " + (std_mats.std_materials.count as string)
            standard_material_rollout.total_label.text = message
            
            standard_material_rollout.material_listbox.items = std_mats.std_material_names
        ) --end fn
        
        /***********************************************************************
        Refreshes the property names in the property dropdowns
        ***********************************************************************/
        fn refresh_standard_property_dropdowns =
        (
            std_mats.collect_standard_properties()
            
            standard_material_rollout.numeric_properties_dropdown.items = std_mats.std_numeric_properties
            standard_material_rollout.boolean_properties_dropdown.items = std_mats.std_boolean_properties
            standard_material_rollout.color_properties_dropdown.items = std_mats.std_color_properties
            standard_material_rollout.map_properties_dropdown.items = std_mats.std_map_properties
            standard_material_rollout.map_multiplier_dropdown.items = std_mats.std_map_multiplier_properties
            standard_material_rollout.map_boolean_dropdown.items = std_mats.std_map_boolean_properties
        ) --end fn
        
        /***********************************************************************
        Returns 'OK' if the user selection and mode are valid. Error message
        otherwise
        ***********************************************************************/
        fn get_validation =
        (
            local user_selection = standard_material_rollout.material_listbox.selection as array
            local validation = "OK"
            
            case standard_material_rollout.mode_radio.state of
            (
                (1): if user_selection.count == 0 do validation = "ERROR: No materials selected"
                (2): if selection.count == 0 do validation = "ERROR: No objects selected"
                (3): if std_mats.std_materials.count == 0 do validation = "ERROR: No standard materials found"
            ) --end case
            
            validation --return value
        ) --end fn
        
        /***********************************************************************
        UI creation
        ***********************************************************************/
        label total_label "Total: 0" pos:[5,5] width:100 height:15
        button update_button "Update" pos:[245,3] width:50 height:15
        multiListBox material_listbox "" pos:[5,20] width:290 height:17
        
        groupBox mode_group "Mode" pos:[5,250] width:290 height:55
            radioButtons mode_radio "" pos:[15,270] width:174 height:32 labels:#("List Select", "Scene Select", "All Materials") columns:2 default:1
            
        groupBox property_group "Properties" pos:[5,310] width:290 height:200
            dropDownList numeric_properties_dropdown "" pos:[15,330] width:200 height:21
            spinner numeric_properties_spinner "" pos:[225,332] width:60 height:15 range:[0,99999,1] type:#float
            dropDownList boolean_properties_dropdown "" pos:[15,360] width:200 height:21
            button boolean_true_button "ON" pos:[225,360] width:30 height:20
            button boolean_false_button "OFF" pos:[255,360] width:30 height:20
            dropDownList color_properties_dropdown "" pos:[15,390] width:200 height:21
            colorPicker color_properties_picker "" pos:[220,390] width:65 height:20
            dropDownList map_properties_dropdown "" pos:[15,420] width:200 height:21
            mapButton map_properties_button "M" pos:[225,420] width:60 height:20
            dropDownList map_multiplier_dropdown "" pos:[15,450] width:200 height:21
            spinner map_multiplier_spinner "" pos:[225,450] width:60 height:15 range:[0,99999,1] type:#float
            dropDownList map_boolean_dropdown "" pos:[15,480] width:200 height:21
            button map_boolean_true_button "ON" pos:[225,480] width:30 height:20
            button map_boolean_false_button "OFF" pos:[255,480] width:30 height:20
        
        label status_label "INFO: Standing by..." pos:[5,520] width:290 height:15
        
        /***********************************************************************
        Rollout open event handler. Prepares the rollout for first time open
        ***********************************************************************/
        on standard_material_rollout open do
        (
            if (classOf Standardmaterial) != material do
            (
                enable_controls false
                standard_material_rollout.status_label.text = "ERROR: Standard material not loaded. Check plugin status"
                return false
            ) --end if
            
            refresh_standard_material_list()
            refresh_standard_property_dropdowns()
            enable_controls true
        ) --end on
        
        /***********************************************************************
        Rollout moved event handler. Stores the current position of the rollout
        ***********************************************************************/
        on standard_material_rollout moved pos do
            setIniSetting std_material_ini "Dialog" "Position" (pos as string)
        
        /***********************************************************************
        Update button clicked event handler. Updates the UI
        ***********************************************************************/
        on update_button pressed do
        (
            if (classOf Standardmaterial) != material do
            (
                enable_controls false
                standard_material_rollout.status_label.text = "ERROR: Standard material not loaded. Check plugin status"
                return false
            ) --end if
            
            refresh_standard_material_list()
            refresh_standard_property_dropdowns()
            enable_controls true
            
            status_label.text = "INFO: Standard material list updated"
        ) --end on
        
        /***********************************************************************
        Numeric spinner changed event handler. Applies the new numeric value to
        the selected property of the standard materials
        ***********************************************************************/
        on numeric_properties_spinner changed the_value do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                std_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                numeric_properties_dropdown.selected \
                                                the_value
                message = ("INFO: " + numeric_properties_dropdown.selected)
                message += (" set to " + (the_value as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Boolean ON clicked event handler. Turns on the selected property of the
        standard materials
        ***********************************************************************/
        on boolean_true_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                std_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                boolean_properties_dropdown.selected \
                                                true
                message = ("INFO: " + boolean_properties_dropdown.selected)
                message += " set to ON"
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Boolean OFF clicked event handler. Turns off the selected property of the
        standard materials
        ***********************************************************************/
        on boolean_false_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                std_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                boolean_properties_dropdown.selected \
                                                false
                message = ("INFO: " + boolean_properties_dropdown.selected)
                message += " set to OFF"
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Color picker clicked event handler. Changes the color of the selected
        property of the standard materials
        ***********************************************************************/
        on color_properties_picker changed the_color do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                std_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                color_properties_dropdown.selected \
                                                the_color
                message = ("INFO: " + color_properties_dropdown.selected)
                message += (" set to " + (the_color as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map picker clicked event handler. Assigns the map to the selected
        property of the standard materials
        ***********************************************************************/
        on map_properties_button picked the_map do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                std_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_properties_dropdown.selected \
                                                the_map
                message = ("INFO: " + map_properties_dropdown.selected)
                message += (" set to " + ((classOf the_map) as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map spinner changed event handler. Applies the new numeric value to
        the selected property of the standard materials
        ***********************************************************************/
        on map_multiplier_spinner changed the_value do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                std_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_multiplier_dropdown.selected \
                                                the_value
                message = ("INFO: " + map_multiplier_dropdown.selected)
                message += (" set to " + (the_value as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map boolean ON clicked event handler. Turns on the selected property of
        the standard materials
        ***********************************************************************/
        on map_boolean_true_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                std_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_boolean_dropdown.selected \
                                                true
                message = ("INFO: " + map_boolean_dropdown.selected)
                message += " set to ON"
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map boolean OFF clicked event handler. Turns off the selected property
        of the standard materials
        ***********************************************************************/
        on map_boolean_false_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                std_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_boolean_dropdown.selected \
                                                false
                message = ("INFO: " + map_boolean_dropdown.selected)
                message += " set to OFF"
            ) --end if
            
            status_label.text = message
        ) --end on
    ) --end rollout
    
    std_material_position = execute (getIniSetting std_material_ini "Dialog" "Position")
    if std_material_position == OK do std_material_position = [100,100]
    if std_material_position.x > sysinfo.desktopsize.x - 300 do
        std_material_position.x = sysinfo.desktopsize.x - 305
    if std_material_position.y > sysinfo.desktopsize.y - 595 do
        std_material_position.y = sysinfo.desktopsize.y - 595
    if std_material_position.x < 0 do std_material_position.x = 0
    if std_material_position.y < 0 do std_material_position.y = 0
    createDialog standard_material_rollout 300 535 std_material_position.x std_material_position.y style:#(#style_titlebar, #style_minimizebox, #style_border, #style_sysmenu)
) --end scope